@@ -236,9 +236,11 @@ createBaseline <- function(description="",
 #'
 #' @examples
 #' \donttest{
-#' # Subset example data
+#' # Subset example data as a demo
 #' data(ExampleDb, package="alakazam")
 #' db <- subset(ExampleDb, c_call == "IGHG" & sample_id == "+7d")
+#' set.seed(112)
+#' db <- dplyr::slice_sample(db, n=100)
 #' 
 #' # Make Baseline object
 #' baseline <- calcBaseline(db, 
@@ -431,10 +433,12 @@ calcBaselineBinomialPdf <- function (x=3,
 #'  }
 #' 
 #' @examples  
-#' \donttest{
-#' # Subset example data from alakazam
+#' \dontrun{
+#' # Subset example data from alakazam as a demo
 #' data(ExampleDb, package="alakazam")
 #' db <- subset(ExampleDb, c_call %in% c("IGHM", "IGHG"))
+#' set.seed(112)
+#' db <- dplyr::slice_sample(db, n=200)
 #' 
 #' # Collapse clones
 #' db <- collapseClones(db, cloneColumn="clone_id",
@@ -465,7 +469,6 @@ calcBaselineBinomialPdf <- function (x=3,
 #' plotBaselineDensity(grouped2, idColumn="sample_id", groupColumn="c_call",
 #'                     colorElement="group", colorValues=isotype_colors,
 #'                     sigmaLimits=c(-1, 1))
-#' 
 #' # Collapse previous isotype (within variable) grouped PDFs into sample PDFs
 #' grouped3 <- groupBaseline(grouped2, groupBy="sample_id")
 #' sample_colors <- c("-1h"="steelblue", "+7d"="firebrick")
@@ -506,6 +509,7 @@ groupBaseline <- function(baseline, groupBy, nproc=1) {
     # export all nesseary environment variables, functions and packages.  
     baseline@db <- data.frame()
     gc()
+    
     if (nproc > 1){        
         cluster <- parallel::makeCluster(nproc, type = "PSOCK")
         parallel::clusterExport( cluster, list('baseline', 'uniqueGroupsIdx',
@@ -566,6 +570,7 @@ groupBaseline <- function(baseline, groupBy, nproc=1) {
                             })
                 rm(matrix_GroupPdfs)
                 gc()
+                
                 # Determine the number of sequences that went into creating each of the PDFs
                 # If running groupBaseline for the first time after calcBaseline, then
                 # each PDF should have a numbOfSeqs=1. 
@@ -600,6 +605,7 @@ groupBaseline <- function(baseline, groupBy, nproc=1) {
                     sorted_numbOfSeqs_region <- sort(numbOfSeqs_region)
                     rm(numbOfSeqs_region)
                     gc()
+                    
                     sorted_list_GroupPdfs <- list()
                     for(newIndex in 1:len_numbOfSeqs_region){
                         sorted_list_GroupPdfs[[newIndex]] <-  list_GroupPdfs[[ as.numeric(names(sorted_numbOfSeqs_region)[newIndex]) ]]
@@ -620,7 +626,7 @@ groupBaseline <- function(baseline, groupBy, nproc=1) {
                         list_sameWeightPdfs <- sorted_list_GroupPdfs[indexesOfWeight]
                         updatedPdf <- groupPosteriors(list_sameWeightPdfs)
                         rm(list_sameWeightPdfs)
-                        gc()
+                        
                         # The new updated weights for this convoluted PDF
                         updatedWeight <- as.numeric(pdfWeight) * length(indexesOfWeight)
                         
@@ -628,7 +634,6 @@ groupBaseline <- function(baseline, groupBy, nproc=1) {
                         sorted_numbOfSeqs_region  <- sorted_numbOfSeqs_region[-indexesOfWeight]
                         sorted_list_GroupPdfs <- sorted_list_GroupPdfs[-indexesOfWeight]
                         rm(indexesOfWeight)
-                        gc()
                         
                         # add the convoluted PDF and its new weight
                         newLength <- length(sorted_numbOfSeqs_region)+1
@@ -784,6 +789,8 @@ groupBaseline <- function(baseline, groupBy, nproc=1) {
 #' # Subset example data
 #' data(ExampleDb, package="alakazam")
 #' db <- subset(ExampleDb, c_call == "IGHG")
+#' set.seed(112)
+#' db <- dplyr::slice_sample(db, n=100)
 #' 
 #' # Collapse clones
 #' db <- collapseClones(db, cloneColumn="clone_id",
@@ -926,9 +933,11 @@ summarizeBaseline <- function(baseline, returnType=c("baseline", "df"), nproc=1)
 #' 
 #' @examples
 #' \donttest{
-#' # Subset example data
+#' # Subset example data as a demo
 #' data(ExampleDb, package="alakazam")
-#' db <- subset(ExampleDb, c_call %in% c("IGHM", "IGHG", "IGHA"))
+#' db <- subset(ExampleDb, c_call %in% c("IGHM", "IGHG"))
+#' set.seed(112)
+#' db <- dplyr::slice_sample(db, n=200)
 #'
 #' # Collapse clones
 #' db <- collapseClones(db, cloneColumn="clone_id",
@@ -1171,10 +1180,12 @@ baseline2DistPValue <-function(base1, base2) {
 #' @seealso  Takes as input a \link{Baseline} object returned from \link{groupBaseline}.
 #' 
 #' @examples
-#' \donttest{
-#' # Subset example data
+#' \dontrun{
+#' # Subset example data as a demo
 #' data(ExampleDb, package="alakazam")
 #' db <- subset(ExampleDb, c_call %in% c("IGHM", "IGHG"))
+#' set.seed(112)
+#' db <- dplyr::slice_sample(db, n=100)
 #' 
 #' # Collapse clones
 #' db <- collapseClones(db, cloneColumn="clone_id",
@@ -1217,7 +1228,7 @@ plotBaselineDensity <- function(baseline, idColumn, groupColumn=NULL, colorEleme
     # baseline=grouped
     # idColumn="sample_id"; groupColumn="c_call"; subsetRegions=NULL; sigmaLimits=c(-5, 5)
     # facetBy="region"; style="density"; size=1; silent=FALSE
-    
+
     # Check input
     colorElement <- match.arg(colorElement)
     style <- match.arg(style)
@@ -1229,8 +1240,8 @@ plotBaselineDensity <- function(baseline, idColumn, groupColumn=NULL, colorEleme
         theme(panel.background=element_blank(),
               panel.grid.major=element_blank(),
               panel.grid.minor=element_blank(),
-              panel.border=element_rect(color="black", size=0.5)) +
-        theme(strip.background=element_rect(fill="white", color="black", size=0.5))
+              panel.border=element_rect(color="black", linewidth = 0.5)) +
+        theme(strip.background=element_rect(fill="white", color="black", linewidth=0.5))
     
     if (style == "density") {
         # Check for proper grouping
@@ -1311,18 +1322,19 @@ plotBaselineDensity <- function(baseline, idColumn, groupColumn=NULL, colorEleme
         names(size_values) <- size_names
         
         # Plot probability density curve
-        p1 <- ggplot(dens_df, aes_string(x="SIGMA", y="DENSITY")) +
+        p1 <- ggplot(dens_df, aes(x=!!rlang::sym("SIGMA"), y=!!rlang::sym("DENSITY"))) +
             base_theme + 
             xlab(expression(Sigma)) +
             ylab("Density") +
-            geom_line(aes(size=size))
+            geom_line(aes(linewidth=!!rlang::sym("size"))) +
+            scale_discrete_manual("linewidth", values = size_values)
         # Add line
         if (colorElement == "id" & is.null(secondaryColumn)) {
-            p1 <- p1 + aes_string(color=idColumn)
+            p1 <- p1 + aes(color=!!rlang::sym(idColumn))
         } else if (colorElement == "id" & !is.null(secondaryColumn)) {
-            p1 <- p1 + aes_string(color=idColumn, linetype=secondaryColumn)
+            p1 <- p1 + aes(color=!!rlang::sym(idColumn), linetype=!!rlang::sym(secondaryColumn))
         } else if (colorElement == "group") {
-            p1 <- p1 + aes_string(color=secondaryColumn, linetype=idColumn)
+            p1 <- p1 + aes(color=!!rlang::sym(secondaryColumn), linetype=!!rlang::sym(idColumn))
         } else {
             stop("Incompatible arguments for groupColumn, colorElement and facetBy")
         }
@@ -1348,20 +1360,20 @@ plotBaselineDensity <- function(baseline, idColumn, groupColumn=NULL, colorEleme
     
     if (sizeElement == "none") {
         p1 <- p1 +
-            guides(size="none", 
-                   colour = guide_legend(override.aes=list(size=size_values)))        
+            guides(linewidth="none", 
+                   colour = guide_legend(override.aes=list(linewidth=size_values)))        
         if (length(unique(c(groupColumn, idColumn))) > 1) {
             p1 <- p1 +
-                guides (linetype=guide_legend(override.aes=list(size=size_values)))
+                guides (linetype=guide_legend(override.aes=list(linewidth=size_values)))
         }
     } else if (sizeElement == colorElement) {
         p1 <- p1 +
-            guides(size="none", 
-                   colour = guide_legend(override.aes = list(size = size_values)))
+            guides(linewidth="none", 
+                   colour = guide_legend(override.aes = list(linewidth = size_values)))
     } else {
         p1 <- p1 +
-            guides(size="none", 
-                   linetype = guide_legend(override.aes = list(size = size_values))) 
+            guides(linewidth="none", 
+                   linetype = guide_legend(override.aes = list(linewidth = size_values))) 
     }
     
       
@@ -1425,9 +1437,11 @@ plotBaselineDensity <- function(baseline, idColumn, groupColumn=NULL, colorEleme
 #' 
 #' @examples
 #' \donttest{
-#' # Subset example data
+#' # Subset example data as a demo
 #' data(ExampleDb, package="alakazam")
 #' db <- subset(ExampleDb, c_call %in% c("IGHM", "IGHG"))
+#' set.seed(112)
+#' db <- dplyr::slice_sample(db, n=25)
 #' 
 #' # Collapse clones
 #' db <- collapseClones(db, cloneColumn="clone_id",
@@ -1452,10 +1466,7 @@ plotBaselineDensity <- function(baseline, idColumn, groupColumn=NULL, colorEleme
 #' isotype_colors <- c("IGHM"="darkorchid", "IGHD"="firebrick", 
 #'                     "IGHG"="seagreen", "IGHA"="steelblue")
 #' plotBaselineSummary(grouped, "sample_id", "c_call", 
-#'                     groupColors=isotype_colors)
-#' 
-#' # Facet by group instead of region
-#' plotBaselineSummary(grouped, "sample_id", "c_call", facetBy="group")
+#'                     groupColors=isotype_colors, facetBy="region")
 #' }
 #' 
 #' @export
@@ -1499,8 +1510,8 @@ plotBaselineSummary <- function(baseline, idColumn, groupColumn=NULL, groupColor
         theme(panel.background=element_blank(),
               panel.grid.major=element_blank(),
               panel.grid.minor=element_blank(),
-              panel.border=element_rect(color="black", size=0.5)) +
-        theme(strip.background=element_rect(fill="white", color="black", size=0.5)) +
+              panel.border=element_rect(color="black", linewidth=0.5)) +
+        theme(strip.background=element_rect(fill="white", color="black", linewidth=0.5)) +
         theme(axis.title.x=element_blank(),
               axis.text.x=element_blank(), 
               axis.ticks.x=element_blank()) +
@@ -1513,14 +1524,17 @@ plotBaselineSummary <- function(baseline, idColumn, groupColumn=NULL, groupColor
         if (!is.null(groupColumn) & !is.null(groupColors)) {
             stats_df[,groupColumn] <- factor(stats_df[,groupColumn], levels=names(groupColors))
         }
-        p1 <- ggplot(stats_df, aes_string(x=idColumn, y="baseline_sigma", ymax=max("baseline_sigma"))) +
+        p1 <- ggplot(stats_df, aes(x=!!rlang::sym(idColumn), 
+                                   y=!!rlang::sym("baseline_sigma"), 
+                                   ymax=max(!!rlang::sym("baseline_sigma")))) +
             base_theme + 
             xlab("") +
             ylab(expression(Sigma)) +
-            geom_hline(yintercept=0, size=1*size, linetype=2, color="grey") +
+            geom_hline(yintercept=0, linewidth=1*size, linetype=2, color="grey") +
             geom_point(size=3*size, position=position_dodge(0.6)) +
-            geom_errorbar(aes_string(ymin="baseline_ci_lower", ymax="baseline_ci_upper"), 
-                          width=0.2, size=0.5*size, alpha=0.8, position=position_dodge(0.6))
+            geom_errorbar(aes(ymin=!!rlang::sym("baseline_ci_lower"), 
+                              ymax=!!rlang::sym("baseline_ci_upper")), 
+                          width=0.2, linewidth=0.5*size, alpha=0.8, position=position_dodge(0.6))
         if (!is.null(title)) {
             p1 <- p1 + ggtitle(title)
         }    
@@ -1529,12 +1543,12 @@ plotBaselineSummary <- function(baseline, idColumn, groupColumn=NULL, groupColor
         } else if (!is.null(groupColumn) & !is.null(groupColors) & facetBy == "region") {
             #groupColors <- factor(groupColors, levels=groupColors)
             p1 <- p1 + scale_color_manual(name=groupColumn, values=groupColors) +
-                aes_string(color=groupColumn) + facet_grid(region ~ .)
+                aes(color=!!rlang::sym(groupColumn)) + facet_grid(region ~ .)
         } else if (!is.null(groupColumn) & is.null(groupColors) & facetBy == "region") {
-            p1 <- p1 + aes_string(color=groupColumn) + facet_grid(region ~ .)
+            p1 <- p1 + aes(color=!!rlang::sym(groupColumn)) + facet_grid(region ~ .)
         } else if (!is.null(groupColumn) & facetBy == "group") {
             p1 <- p1 + scale_color_manual(name="Region", values=REGION_PALETTE) +
-                aes_string(color="region") + facet_grid(paste(groupColumn, "~ ."))
+                aes(color=!!rlang::sym("region")) + facet_grid(paste(groupColumn, "~ ."))
         } else {
             stop("Cannot facet by group if groupColumn=NULL")
         }
@@ -1984,7 +1998,7 @@ calcBaseline <- function(db,
             cluster, list('cols_observed', 'cols_expected','calcBaselineHelper'), 
             envir=environment() 
         )
-        registerDoParallel(cluster)
+        registerDoParallel(cluster,cores=nproc)
     }
     
     list_pdfs <- list()
